#version 310 es
precision highp float;
/**
* MDL定制特效渲染片段着色程序
* Created by wgs on 21-11-12.
*/
//参数介绍
uniform sampler2D text_source;
uniform bool needDark; //是否开启暗角
uniform bool needNoise; //是否开启噪点
uniform float frameNumberCount; //当前帧数
uniform float noiseDensity; //噪点密度
uniform float blockCount; //分块总数
uniform float blockOffset; //偏移量(当前属于第几块偏移量)
in vec2 vTexCoord;
out vec4 outColor;


float nrand( vec2 n ) {
    return fract(sin(dot(n.xy, vec2(12.9898, 78.233)))* 43758.5453);
}


float n6rand(vec2 n)
{
    float t = frameNumberCount * (30.0/60.0)/100.0;

    float nrnd0 = nrand( n + 0.07*t );
    float nrnd1 = nrand( n + 0.11*t );
    float nrnd2 = nrand( n + 0.13*t );
    float nrnd3 = nrand( n + 0.17*t );
    float nrnd4 = nrand( n + 0.19*t );
    float nrnd5 = nrand( n + 0.23*t );
    return (nrnd0+nrnd1+nrnd2+nrnd3+nrnd4+nrnd5) / 6.0;
}

float lum(vec3 color) {
    return color.x * 0.299 + color.y *0.114 + color.z * 0.587;
}

vec3 colorNoise(vec3 color, vec2 tc) {
    float its = n6rand(tc);
    its = its * 2.0;
    float brightness = lum(color);
    vec3 noise = color * its;
    float noiseStrength = noiseDensity;
    noiseStrength = noiseStrength * (1.0 - brightness);
    noiseStrength = step(0.0,noiseStrength) * noiseStrength;
    vec3 withNoise = mix(color,noise,noiseStrength);
    return vec3(withNoise);
}

vec3 colorDark(vec3 color, vec2 uv) {
    vec3 factor = vec3(0.2125, 0.7154, 0.0721);
    float Y = dot(color, factor);
    float count = blockCount;
    float offset = blockOffset;
    uv.x = (uv.x - 0.5) * 0.75 + 0.5;
    //拍照做分块处理
    if (blockCount > 0.0) {
        uv.y = uv.y / count + offset / count;
    }
    float dist = distance(uv, vec2(0.5,0.5));
    float pow = 1.0 / pow(100.0, (1.0 - Y));
    return (color * (1.0 - pow) * smoothstep(1.4, 0.2, dist) + color * pow);
}

vec4 processColor(vec2 uv) {
    //获取输入像素
    vec3 color = texture(text_source, uv, 0.0).rgb;
    //处理噪声
    if (needNoise) {
        color = colorNoise(color, uv);
    }
    //处理暗角
    if (needDark) {
        color = colorDark(color, uv);
    }
    return vec4(color, 1.0);
}

void main() {
    vec4 finalColor = processColor(vTexCoord);
    outColor = finalColor;
}